/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */

#include "config.h"
#include <stdio.h>

#include <Xm/DialogS.h>
#include <Xm/PushB.h>
#include <Xm/Form.h>

#include "xmap.h"
#include "debug.h"


extern Widget toplevel;

static Widget pickdirdialog = NULL;
static Widget form;

static Dimension width, height;

static void build_pickdir_dialog(Widget w, Widget formw);

static void pushed(Widget w, XtPointer cd, XmPushButtonCallbackStruct cbs)
{
//    fprintf(stderr, "Button %d pushed.\n", (int)cd);

    switch ((int)cd) {
	case 0: // NW
	    load_map(map.nwmap);
	    break;
	case 1: // N
	    load_map(map.nmap);
	    break;
	case 2: // NE
	    load_map(map.nemap);
	    break;
	case 3: // W
	    load_map(map.wmap);
	    break;
	case 4: // EXIT
	    XtPopdown(pickdirdialog);
	    XtUnmanageChild(pickdirdialog);
	    return;
	case 5: // E
	    load_map(map.emap);
	    break;
	case 6: // SW
	    load_map(map.swmap);
	    break;
	case 7: // S
	    load_map(map.smap);
	    break;
	case 8: // SE
	    load_map(map.semap);
	    break;
    }
  XtDestroyWidget(w);
  build_pickdir_dialog(w, form);
}

static int have_map(int n_map)
{
    switch (n_map) {
	case 0: // NW
	    return (int) *map.nwmap;
	    break;
	case 1: // N
	    return (int) *map.nmap;
	    break;
	case 2: // NE
	    return (int) *map.nemap;
	    break;
	case 3: // W
	    return (int) *map.wmap;
	    break;
	case 4: // EXIT
	    return TRUE;
	    break;
	case 5: // E
	    return (int) *map.emap;
	    break;
	case 6: // SW
	    return (int) *map.swmap;
	    break;
	case 7: // S
	    return (int) *map.smap;
	    break;
	case 8: // SE
	    return (int) *map.semap;
	    break;
    }
    return FALSE;
}

void save_pickdir_properties()
{
	save_geometry(pickdirdialog, "pickdir");
}

void do_pickdir_dialog(Widget w, XtPointer client_data, XtPointer call_data)
{
    Arg args[100];
    XmString s;
    int n;
    Position x, y;
    char bufp[255];

    if (!pickdirdialog) {
	XGCValues gcv;
	int iw, ih;

	pickdirdialog = XtVaCreatePopupShell("pickdirdialog",
				xmDialogShellWidgetClass, GetTopShell(w),
					    XmNtitle, "...", /* give arbitrary title in wm */
					    XmNminWidth, 60,
					    XmNminHeight, 60,
					    XmNmaxWidth, 190,
					    XmNmaxHeight, 190,
					    NULL);

	XtVaGetValues(toplevel,
		      XmNx, &x,
		      XmNy, &y,
		      NULL);

	y += 50;
	snprintf(bufp, 255, "%dx%d+%d+%d", 160, 300, x, y);
	GetPrivateProfileString("Layout", "pickdir", bufp, bufp, 255, inifile);

	XtVaSetValues(pickdirdialog, XmNgeometry, bufp, NULL);

	sscanf(bufp, "%dx%d", &iw, &ih);

    	form = XtVaCreateWidget("form", xmFormWidgetClass, pickdirdialog,
				XmNfractionBase, 3,
				XmNautoUnmanage, False,
				NULL);
	build_pickdir_dialog(w, form);
#if 0
	XtVaSetValues(pickdirdialog,
		      XmNx, x,
		      XmNy, y,
		      XmNwidth, width,
		      XmNheight, height,
		      NULL);
#endif
    }
    XtManageChild(form);
    XtManageChild(pickdirdialog);
    XtPopup(pickdirdialog, XtGrabNone);
}

static void build_pickdir_dialog(Widget w, Widget formw)
{
    Position x, y;

    for (x=0; x<3; x++) {
	for (y=0; y<3; y++) {
	    w = XtVaCreateManagedWidget(" ",
	        xmPushButtonWidgetClass, formw,
		XmNtopAttachment, XmATTACH_POSITION,
		XmNtopPosition, y,
		XmNleftAttachment, XmATTACH_POSITION,
		XmNleftPosition, x,
		XmNrightAttachment, XmATTACH_POSITION,
		XmNrightPosition, x+1,
		XmNbottomAttachment, XmATTACH_POSITION,
		XmNbottomPosition, y+1,
	        XmNlabelType, XmPIXMAP,
		XmNlabelPixmap, 
		get_darrow_pixmap( 3*y+x + 
				   ((have_map(3*y+x) == FALSE) ? 9 : 0)),
		XmNhighlightThickness, 0,
//	        XmNshadowThickness, 0,
	        XmNpushButtonEnabled, have_map(3*y+x),
//	        XmNshadowType, XmSHADOW_ETCHED_IN,
		NULL);
	    XtAddCallback(w, XmNactivateCallback, pushed, (void *)(3*y+x));
	}
    }
}
