/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
#include "config.h"

#include <stdio.h>
#include <assert.h>
#include <Xm/DialogS.h>
#include <Xm/ScrolledW.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <Xm/TextF.h>
#include <Xm/Label.h>
#include <Xm/Frame.h>
#include <Xm/BulletinB.h>
#include <Xm/SelectioB.h>

#include "ComboBox.h"
#include "Tab.h"
#include "xmap.h"
#include "gps.h"
#include "debug.h"

#define GOOD_VASIMPLE 0	// Lesstif is broken. Set to "1" when fixed.

#define OCHX 245	// OK, Cancel, Help x position

extern struct DATUM const gDatum[];
extern struct ELLIPSOID const gEllipsoid[];
extern Widget toplevel;
extern Widget draw_w;
static Widget selectdialog = NULL;
static Widget dialog;
static Widget tab;
static Widget bul1, bul2, bul3, bul4, bul5;
static Widget portw, server, tow, intw;
static Widget start;
static Widget cancel;
static Widget help;
static Widget datum;
static Widget ellips;
static Widget printc, fname;

static int format;
static int nunits;
static int ndatum;
static int nellips;
static int nncol;
static int useutc;
static int pdebug;

static Widget proximity;
static int ncolor, naction, nprox, nproxu;

static Pixel background, grey;

extern void show_help_cb();

char *a_units[] = {
	"m",
	"Km",
	"feet",
	"yards",
	"miles",
	"nm",
	NULL
};

char *r_colors[] = {
	"None",
	"Black",
	"White",
	"Red",
	"Green",
	"Blue",
	NULL
};

char *r_actions[] = {
	"Popup alert Dialog",
	"Popup and Beep 1 time",
	"Popup and Beep 5 times",
	"Popup and Beep 10 times",
	"Popup and Beep till ack.",
	"Beep 1 time",
	"Beep 5 times",
	"Beep 10 times",
	NULL
};

int paper_orientation, paper_size;
int print_destination;

static void ok_cb(Widget w, XtPointer client_data, XmAnyCallbackStruct * cbs)
{
    char bufp[128];
    int gpsint = strtol(XmTextFieldGetString(intw), (char **)NULL, 10);

    options.format = format;
    options.nicecol = nncol;
    options.useutc = useutc;
    options.pdebug = pdebug;

    WritePrivateProfileInt("General", "ddd", format, inifile);

    snprintf(bufp, 127, "%s:%s", XmTextFieldGetString(server), XmTextFieldGetString(portw));
    WritePrivateProfileInt("General", "gpsinterval", gpsint, inifile);
    WritePrivateProfileString("Servers", "server1", bufp, inifile);

    WritePrivateProfileString("General", "units", unit_names[nunits], inifile);
    unit = nunits;

    WritePrivateProfileString("General", "nicecol", on_off[nncol], inifile);

    WritePrivateProfileString("General", "useutc", on_off[useutc], inifile);

    WritePrivateProfileInt("Maps", "datum", ndatum, inifile);
    WritePrivateProfileInt("Maps", "ellipsoid", nellips, inifile);
    assert(nellips < 23);

    WritePrivateProfileInt("Printing", "papersize", paper_size, inifile);
    WritePrivateProfileString("Printing", "destination",
	print_destination?"file":"printer", inifile);
    WritePrivateProfileString("Printing", "orientation",
	paper_orientation?"landscape":"portrait", inifile);
    WritePrivateProfileString("Printing", "printcommand",
	XmTextFieldGetString(printc), inifile);
    WritePrivateProfileString("Printing", "filename",
	XmTextFieldGetString(fname), inifile);

    WritePrivateProfileInt("Routes", "color", ncolor, inifile);
    WritePrivateProfileInt("Routes", "action", naction, inifile);
    WritePrivateProfileString("Routes", "proximity",
	XmTextFieldGetString(proximity), inifile);
    WritePrivateProfileInt("Routes", "units", nproxu, inifile);

    XtPopdown(selectdialog);
    WriteOutProfiles();
}


static void cancel_cb(Widget w, XtPointer client_data, XmAnyCallbackStruct * cbs)
{
    XtPopdown(selectdialog);
}

static void datum_cb(Widget w, XtPointer client_data, MyComboBoxCallbackStruct * cbs)
{
    XmString s;
    ndatum = cbs->item_position;
    nellips = gDatum[ndatum].ellipsoid;

    assert(nellips < 23);

    s = XmStringCreateSimple(gEllipsoid[nellips].name);
    MyComboBoxSetItem(ellips, s);
    XmStringFree(s);
}

static void ellips_cb(Widget w, XtPointer client_data, MyComboBoxCallbackStruct * cbs)
{
    XmString s;
    nellips = cbs->item_position;
    assert(nellips < 23);
}

static void color_cb(Widget w, XtPointer client_data, MyComboBoxCallbackStruct * cbs)
{
    XmString s;
    ncolor = cbs->item_position;
    assert(ncolor < 6);
}

static void action_cb(Widget w, XtPointer client_data, MyComboBoxCallbackStruct * cbs)
{
    XmString s;
    naction = cbs->item_position;
    assert(naction < 8);
}

static void proxu_cb(Widget w, XtPointer client_data, MyComboBoxCallbackStruct * cbs)
{
    XmString s;
    nproxu = cbs->item_position;
    assert(nproxu < 6);
}

static void toggle_cb(Widget w, int n, XmToggleButtonCallbackStruct * cbs)
{
    if (cbs->set == True)
	format = n;
}

static void unit_cb(Widget w, int n, XmToggleButtonCallbackStruct * cbs)
{
    if (cbs->set == True) nunits = n + 1;
}

static void nncol_cb(Widget w, XtPointer client_data, XmToggleButtonCallbackStruct * cbs)
{
    nncol = cbs->set;
}

static void useutc_cb(Widget w, int n, XmToggleButtonCallbackStruct * cbs)
{
    if (cbs->set == True) useutc = n?0:1;
}

static void pdebug_cb(Widget w, XtPointer client_data, XmToggleButtonCallbackStruct * cbs)
{
    pdebug = cbs->set;
}

static pf_toggle()
{
    if (!print_destination) {
	XtVaSetValues(fname,
		XmNcursorPositionVisible, False,
		XmNsensitive, False,
		NULL);
	XtVaSetValues(printc,
		XmNcursorPositionVisible, True,
		XmNsensitive, True,
		NULL);
    }
    else {
	XtVaSetValues(printc,
		XmNcursorPositionVisible, False,
		XmNsensitive, False,
		NULL);
	XtVaSetValues(fname,
		XmNcursorPositionVisible, True,
		XmNsensitive, True,
		NULL);
    }
}

static void psize_cb(Widget w, int n, XmToggleButtonCallbackStruct * cbs)
{
    if (cbs->set == True)
	paper_size = n;
}

static void porientation_cb(Widget w, int n, XmToggleButtonCallbackStruct * cbs)
{
    if (cbs->set == True)
	paper_orientation = n;
}

static void pdestination_cb(Widget w, int n, XmToggleButtonCallbackStruct * cbs)
{
    if (cbs->set == True)
	print_destination = n;
    pf_toggle();
}

static void do_printing(Widget bul)
{
    Arg args[10];
    int n;
    XmString b1, b2, b3, b4;
    Widget tb, toggle_dest, toggle_orientation, toggle_size;
    char temp[128];

    XtVaCreateManagedWidget("Printing:", xmLabelWidgetClass, bul,
			    XmNx, 40,
			    XmNy, 15,
			    NULL);

    XtVaCreateManagedWidget("Print Command:", xmLabelWidgetClass, bul,
			    XmNx, 10,
			    XmNy, 45,
			    NULL);

    XtVaCreateManagedWidget("File Name:", xmLabelWidgetClass, bul,
			    XmNx, 34,
			    XmNy, 75,
			    NULL);

    XtVaCreateManagedWidget("Orientation:", xmLabelWidgetClass, bul,
			    XmNx, 22,
			    XmNy, 110,
			    NULL);

    XtVaCreateManagedWidget("Paper Size:", xmLabelWidgetClass, bul,
			    XmNx, 28,
			    XmNy, 140,
			    NULL);

    b1 = XmStringCreateSimple(" Include error handler");
    tb = XtVaCreateManagedWidget("bn", xmToggleButtonWidgetClass, bul,
				 XmNlabelString, b1,
				 XmNset, options.pdebug,
				 XmNselectColor, get_pixel(bul, "red"),
				 XmNx, 110,
				 XmNy, 170,
				 NULL);
    XmStringFree(b1);
    XtAddCallback(tb, XmNvalueChangedCallback, (XtCallbackProc) pdebug_cb, NULL);
    pdebug = options.pdebug;

    b1 = XmStringCreateSimple("Printer");
    b2 = XmStringCreateSimple("File");

    GetPrivateProfileString("Printing", "destination", "file", temp, 128, inifile);
    print_destination = strcmp(temp,"printer")?1:0;

#if GOOD_VASIMPLE
    toggle_dest = XmVaCreateSimpleRadioBox(bul, "dest", 
				print_destination, (XtCallbackProc) pdestination_cb,
				XmNx, 110,
				XmNy, 13,
				XmNorientation, XmHORIZONTAL,
				XmNmarginHeight, 0,
				XmVaRADIOBUTTON, b1, 0, NULL, NULL,
				XmVaRADIOBUTTON, b2, 0, NULL, NULL,
				XmNhighlightThickness, 0,
				NULL);
#else
    {
	Widget one, two;

	n = 0;
	XtSetArg(args[n], XmNorientation, XmHORIZONTAL); n++;
	XtSetArg(args[n], XmNx, 110); n++;
	XtSetArg(args[n], XmNy, 13); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	toggle_dest = XmCreateRadioBox(bul, "dest", args, n);
	one = XtVaCreateManagedWidget("One", xmToggleButtonGadgetClass,
		toggle_dest,
		XmNlabelString, b1,
		XmNset, (print_destination==0),
		NULL);
	XtAddCallback(one, XmNvalueChangedCallback,
		(XtCallbackProc) pdestination_cb, 0);

	two = XtVaCreateManagedWidget("Two", xmToggleButtonGadgetClass,
		toggle_dest,
		XmNlabelString, b2,
		XmNset, (print_destination==1),
		NULL);
	XtAddCallback(two, XmNvalueChangedCallback,
		(XtCallbackProc) pdestination_cb, 1);
	
    }
#endif
    XtManageChild(toggle_dest);
    XmStringFree(b1);
    XmStringFree(b2);

    GetPrivateProfileString("Printing", "printcommand", "lpr", temp, 128, inifile);
    printc = XtVaCreateManagedWidget("pc", xmTextFieldWidgetClass, bul,
	XmNhighlightThickness, 0,
	XmNbackground, background,
	XmNx, 110,
	XmNy, 41,
	XmNwidth, 190,
	XmNvalue, temp,
	NULL);

    GetPrivateProfileString("Printing", "filename", "xmap.ps", temp, 128, inifile);
    fname = XtVaCreateManagedWidget("fn", xmTextFieldWidgetClass, bul,
	XmNhighlightThickness, 0,
	XmNbackground, background,
	XmNx, 110,
	XmNy, 71,
	XmNwidth, 140,
	XmNvalue, temp,
	NULL);

    b1 = XmStringCreateSimple("Portrait");
    b2 = XmStringCreateSimple("Landscape");
    GetPrivateProfileString("Printing", "orientation", "portrait", temp, 128, inifile);
    paper_orientation = strcmp(temp,"portrait")?1:0;

#if GOOD_VASIMPLE
    toggle_orientation = XmVaCreateSimpleRadioBox(bul, "dest", 
				paper_orientation, (XtCallbackProc) porientation_cb,
				XmNx, 110,
				XmNy, 108,
				XmNorientation, XmHORIZONTAL,
				XmNmarginHeight, 0,
				XmVaRADIOBUTTON, b1, 0, NULL, NULL,
				XmVaRADIOBUTTON, b2, 0, NULL, NULL,
				XmNhighlightThickness, 0,
				NULL);
#else
    {
	Widget one, two;

	n = 0;
	XtSetArg(args[n], XmNorientation, XmHORIZONTAL); n++;
	XtSetArg(args[n], XmNx, 110); n++;
	XtSetArg(args[n], XmNy, 108); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	toggle_orientation = XmCreateRadioBox(bul, "dest", args, n);
	one = XtVaCreateManagedWidget("One", xmToggleButtonGadgetClass,
		toggle_orientation,
		XmNlabelString, b1,
		XmNset, (paper_orientation==0),
		NULL);
	XtAddCallback(one, XmNvalueChangedCallback,
		(XtCallbackProc) porientation_cb, 0);

	two = XtVaCreateManagedWidget("Two", xmToggleButtonGadgetClass,
		toggle_orientation,
		XmNlabelString, b2,
		XmNset, (paper_orientation==1),
		NULL);
	XtAddCallback(two, XmNvalueChangedCallback,
		(XtCallbackProc) porientation_cb, 1);
	
    }
#endif
    XtManageChild(toggle_orientation);
    XmStringFree(b1);
    XmStringFree(b2);

    b1 = XmStringCreateSimple("Letter");
    b2 = XmStringCreateSimple("Legal");
    b3 = XmStringCreateSimple("DIN A4");

    paper_size = GetPrivateProfileInt("Printing", "papersize", 0, inifile);

#ifdef GOOD_VASIMPLE
    toggle_size = XmVaCreateSimpleRadioBox(bul, "dest",
			paper_size, (XtCallbackProc) psize_cb,
			XmNx, 110,
			XmNy, 138,
			XmNorientation, XmHORIZONTAL,
			XmNmarginHeight, 0,
			XmVaRADIOBUTTON, b1, 0, NULL, NULL,
			XmVaRADIOBUTTON, b2, 0, NULL, NULL,
			XmVaRADIOBUTTON, b3, 0, NULL, NULL,
			XmNhighlightThickness, 0,
			NULL);
#else
    {
	Widget one, two, three;

	n = 0;
	XtSetArg(args[n], XmNorientation, XmHORIZONTAL); n++;
	XtSetArg(args[n], XmNx, 110); n++;
	XtSetArg(args[n], XmNy, 138); n++;
	XtSetArg(args[n], XmNhighlightThickness, 0); n++;
	toggle_size = XmCreateRadioBox(bul, "dest", args, n);
	one = XtVaCreateManagedWidget("One", xmToggleButtonGadgetClass,
		toggle_size,
		XmNlabelString, b1,
		XmNset, (paper_size==0),
		NULL);
	XtAddCallback(one, XmNvalueChangedCallback,
		(XtCallbackProc) psize_cb, 0);

	two = XtVaCreateManagedWidget("Two", xmToggleButtonGadgetClass,
		toggle_size,
		XmNlabelString, b2,
		XmNset, (paper_size==1),
		NULL);
	XtAddCallback(two, XmNvalueChangedCallback,
		(XtCallbackProc) psize_cb, 1);
	
	three = XtVaCreateManagedWidget("Three", xmToggleButtonGadgetClass,
		toggle_size,
		XmNlabelString, b3,
		XmNset, (paper_size==2),
		NULL);
	XtAddCallback(two, XmNvalueChangedCallback,
		(XtCallbackProc) psize_cb, 2);
	
    }
#endif
    XtManageChild(toggle_size);
    XmStringFree(b1);
    XmStringFree(b2);
    XmStringFree(b3);

    pf_toggle();
}

static void do_datum(Widget bul)
{
    Arg args[10];
    int n;
    XmString s;

    ndatum = GetPrivateProfileInt("Maps", "datum", 100, inifile);
    nellips = GetPrivateProfileInt("Maps", "ellipsoid", 20, inifile);
    assert(nellips < 23);

    s = XmStringCreateSimple("Map Datum:");
    XtVaCreateManagedWidget("l1", xmLabelWidgetClass, bul,
			    XmNlabelString, s,
			    XmNx, 10,
			    XmNy, 15,
			    NULL);
    XmStringFree(s);

    s = XmStringCreateSimple("Ellipsoid:");
    XtVaCreateManagedWidget("l2", xmLabelWidgetClass, bul,
			    XmNlabelString, s,
			    XmNx, 10,
			    XmNy, 45,
			    NULL);
    XmStringFree(s);

    n = 0;
    XtSetArg(args[n], XmNx, 100);
    n++;
    XtSetArg(args[n], XmNy, 10);
    n++;
    datum = MyCreateComboBox(bul, "datum", args, n);
    XtAddCallback(datum, XmNselectionCallback, (XtCallbackProc) datum_cb, NULL);

    n = 0;
    while (gDatum[n].name) {
	s = XmStringCreateSimple(gDatum[n].name);
	MyComboBoxAddItem(datum, s, 0, True);
	XmStringFree(s);
	n++;
    }

    n = 0;
    XtSetArg(args[n], XmNx, 100);
    n++;
    XtSetArg(args[n], XmNy, 40);
    n++;
    ellips = MyCreateComboBox(bul, "ellips", args, n);
    XtAddCallback(ellips, XmNselectionCallback, (XtCallbackProc) ellips_cb, NULL);

    n = 0;
    while (gEllipsoid[n].name) {
	s = XmStringCreateSimple(gEllipsoid[n].name);
	MyComboBoxAddItem(ellips, s, 0, True);
	XmStringFree(s);
	n++;
    }

    XtVaSetValues(datum, XmNwidth, 180, NULL);
    XtVaSetValues(ellips, XmNwidth, 180, NULL);


    s = XmStringCreateSimple(gDatum[ndatum].name);
    MyComboBoxSetItem(datum, s);
    XmStringFree(s);

    s = XmStringCreateSimple(gEllipsoid[nellips].name);
    MyComboBoxSetItem(ellips, s);
    XmStringFree(s);

    XtManageChild(datum);
    XtManageChild(ellips);
}

char *route_color()
{
    return r_colors[GetPrivateProfileInt("Routes", "color", 2, inifile)];
}

static void do_routes(Widget bul)
{
    Arg args[10];
    XmString s;
    char *p, temp[128];
    int n;
    Widget unit, color, action;

    ncolor = GetPrivateProfileInt("Routes", "color", 2, inifile); // default to red
    naction = GetPrivateProfileInt("Routes", "action", 0, inifile);
    nprox = GetPrivateProfileInt("Routes", "proximity", 150, inifile); 
    nproxu = GetPrivateProfileInt("Routes", "units", 3, inifile); 

    s = XmStringCreateSimple("Proximity Alarm:");
    XtVaCreateManagedWidget("l1", xmLabelWidgetClass, bul,
			    XmNlabelString, s,
			    XmNx, 10,
			    XmNy, 25,
			    NULL);
    XmStringFree(s);

    n = 0;
    XtSetArg(args[n], XmNx, 170); n++;
    XtSetArg(args[n], XmNy, 21); n++;
    unit = MyCreateComboBox(bul, "unit", args, n);
    n = 0;
    while (a_units[n]) {
	s = XmStringCreateSimple(a_units[n]);
	MyComboBoxAddItem(unit, s, 0, True);
	XmStringFree(s);
	n++;
    }
    XtAddCallback(unit, XmNselectionCallback, (XtCallbackProc) proxu_cb, NULL);

    s = XmStringCreateSimple("Default action:");
    XtVaCreateManagedWidget("l2", xmLabelWidgetClass, bul,
			    XmNlabelString, s,
			    XmNx, 10,
			    XmNy, 60,
			    NULL);
    XmStringFree(s);

    s = XmStringCreateSimple("Route color:");
    XtVaCreateManagedWidget("l2", xmLabelWidgetClass, bul,
			    XmNlabelString, s,
			    XmNx, 10,
			    XmNy, 95,
			    NULL);
    XmStringFree(s);

    n = 0;
    XtSetArg(args[n], XmNx, 115); n++;
    XtSetArg(args[n], XmNy, 91); n++;
    color = MyCreateComboBox(bul, "color", args, n);
    n = 0;
    while (r_colors[n]) {
	s = XmStringCreateSimple(r_colors[n]);
	MyComboBoxAddItem(color, s, 0, True);
	XmStringFree(s);
	n++;
    }
    XtAddCallback(color, XmNselectionCallback, (XtCallbackProc) color_cb, NULL);

    n = 0;
    XtSetArg(args[n], XmNx, 115); n++;
    XtSetArg(args[n], XmNy, 55); n++;
    action = MyCreateComboBox(bul, "action", args, n);
    n = 0;
    while (r_actions[n]) {
	s = XmStringCreateSimple(r_actions[n]);
	MyComboBoxAddItem(action, s, 0, True);
	XmStringFree(s);
	n++;
    }
    XtAddCallback(action, XmNselectionCallback, (XtCallbackProc) action_cb, NULL);

    sprintf(temp, "%d", nprox);
    proximity = XtVaCreateManagedWidget("t1", xmTextFieldWidgetClass, bul,
			    XmNhighlightThickness, 0,
			    XmNbackground, background,
			    XmNx, 115,
			    XmNy, 20,
			    XmNwidth, 50,
			    XmNvalue, temp,
			    NULL);

///////
    s = XmStringCreateSimple(a_units[nproxu]);
    MyComboBoxSetItem(unit, s);
    XmStringFree(s);

    s = XmStringCreateSimple(r_colors[ncolor]);
    MyComboBoxSetItem(color, s);
    XmStringFree(s);

    s = XmStringCreateSimple(r_actions[naction]);
    MyComboBoxSetItem(action, s);
    XmStringFree(s);

///////
    XtManageChild(unit);
    XtManageChild(color);
    XtManageChild(action);

}

static void do_gps(Widget bul)
{
    XmString s;
    char *port = DEFAULTPORT;
    char *timeout = "10";
    char *interval = "1000";
    char *p, temp[128];

    GetPrivateProfileString("Servers", "server1", "", temp, 128, inifile);
    if (p = strchr(temp, ':')) {
	if (*(p + 1))
	    port = p + 1;
	*p = '\0';
    }
    s = XmStringCreateSimple("Server:");
    XtVaCreateManagedWidget("l1", xmLabelWidgetClass, bul,
			    XmNlabelString, s,
			    XmNy, 25,
			    NULL);
    XmStringFree(s);

    s = XmStringCreateSimple("Port:");
    XtVaCreateManagedWidget("l2", xmLabelWidgetClass, bul,
			    XmNlabelString, s,
			    XmNy, 60,
			    NULL);
    XmStringFree(s);

    s = XmStringCreateSimple("Timeout:");
    XtVaCreateManagedWidget("l3", xmLabelWidgetClass, bul,
			    XmNlabelString, s,
			    XmNx, 128,
			    XmNy, 60,
			    NULL);
    XmStringFree(s);
    s = XmStringCreateSimple("seconds.");
    XtVaCreateManagedWidget("l4", xmLabelWidgetClass, bul,
			    XmNlabelString, s,
			    XmNx, 215,
			    XmNy, 60,
			    NULL);
    XmStringFree(s);

    s = XmStringCreateSimple("GPS Poll Interval:");
    XtVaCreateManagedWidget("l5", xmLabelWidgetClass, bul,
			    XmNlabelString, s,
			    XmNy, 95,
			    NULL);
    XmStringFree(s);
    s = XmStringCreateSimple("milliseconds.");
    XtVaCreateManagedWidget("l6", xmLabelWidgetClass, bul,
			    XmNlabelString, s,
			    XmNx, 215,
			    XmNy, 95,
			    NULL);
    XmStringFree(s);

    server = XtVaCreateManagedWidget("t1", xmTextFieldWidgetClass, bul,
				     XmNhighlightThickness, 0,
				     XmNbackground, background,
				     XmNx, 65,
				     XmNy, 20,
				     XmNwidth, 200,
				     XmNvalue, temp,
				     NULL);

    portw = XtVaCreateManagedWidget("t2", xmTextFieldWidgetClass, bul,
				    XmNhighlightThickness, 0,
				    XmNbackground, background,
				    XmNx, 65,
				    XmNy, 55,
				    XmNwidth, 50,
				    XmNvalue, port,
				    NULL);


    tow = XtVaCreateManagedWidget("t3", xmTextFieldWidgetClass, bul,
				  XmNhighlightThickness, 0,
				  XmNbackground, background,
				  XmNx, 183,
				  XmNy, 55,
				  XmNwidth, 27,
				  XmNvalue, timeout,
				  NULL);

    intw = XtVaCreateManagedWidget("t4", xmTextFieldWidgetClass, bul,
				   XmNhighlightThickness, 0,
				   XmNbackground, background,
				   XmNx, 169,
				   XmNy, 90,
				   XmNwidth, 41,
				   XmNvalue, interval,
				   NULL);

}

static void do_display(Widget bul1)
{
    XmString b1, b2, b3, b4;
    Widget toggle_box, fr1, fr2, fr3, tb;

    fr1 = XtVaCreateManagedWidget("fr1", xmFrameWidgetClass, bul1,
				  XmNx, 0,
				  XmNy, 10,
				  XmNwidth, 150,
				  XmNheight, 120,
				  NULL);

    fr2 = XtVaCreateManagedWidget("fr2", xmFrameWidgetClass, bul1,
				  XmNx, 180,
				  XmNy, 10,
				  XmNwidth, 110,
				  XmNheight, 70,
				  NULL);

    fr3 = XtVaCreateManagedWidget("fr3", xmFrameWidgetClass, bul1,
				  XmNx, 180,
				  XmNy, 90,
				  XmNwidth, 110,
				  XmNheight, 94,
				  NULL);


    XtVaCreateManagedWidget("Coordinates", xmLabelWidgetClass, fr1,
			    XmNchildType, XmFRAME_TITLE_CHILD,
		      XmNchildHorizontalAlignment, XmALIGNMENT_BEGINNING,
			    NULL);

    b1 = XmStringCreateSimple(" dddmm'ss.s\"");
    b2 = XmStringCreateSimple(" dddmm.mmm'");
    b3 = XmStringCreateSimple(" ddd.ddddd");
    b4 = XmStringCreateSimple(" UTM");

    toggle_box = XmVaCreateSimpleRadioBox(fr1,
		    "format", options.format, (XtCallbackProc) toggle_cb,
				      XmVaRADIOBUTTON, b1, 0, NULL, NULL,
				      XmVaRADIOBUTTON, b2, 0, NULL, NULL,
				      XmVaRADIOBUTTON, b3, 0, NULL, NULL,
				      XmVaRADIOBUTTON, b4, 0, NULL, NULL,
					  XmNhighlightThickness, 0,
					  NULL);

    XmStringFree(b1);
    XmStringFree(b2);
    XmStringFree(b3);
    XmStringFree(b4);

    format = options.format;
    XtManageChild(toggle_box);

    XtVaCreateManagedWidget("Date/Time", xmLabelWidgetClass, fr2,
			    XmNchildType, XmFRAME_TITLE_CHILD,
		      XmNchildHorizontalAlignment, XmALIGNMENT_BEGINNING,
			    NULL);

    b1 = XmStringCreateSimple(" UTC");
    b2 = XmStringCreateSimple(" Local Time");

    useutc = options.useutc;
    toggle_box = XmVaCreateSimpleRadioBox(fr2,
				"time",useutc?0:1, (XtCallbackProc) useutc_cb,
				XmVaRADIOBUTTON, b1, 0, NULL, NULL,
				XmVaRADIOBUTTON, b2, 0, NULL, NULL,
				XmNhighlightThickness, 0,
				NULL);

    XmStringFree(b1);
    XmStringFree(b2);
    XtManageChild(toggle_box);

    XtVaCreateManagedWidget("Units", xmLabelWidgetClass, fr3,
			    XmNchildType, XmFRAME_TITLE_CHILD,
		      XmNchildHorizontalAlignment, XmALIGNMENT_BEGINNING,
			    NULL);


    b1 = XmStringCreateSimple(" Statute");
    b2 = XmStringCreateSimple(" Metric");
    b3 = XmStringCreateSimple(" Nautical");

    toggle_box = XmVaCreateSimpleRadioBox(fr3,
			     "units", unit - 1, (XtCallbackProc) unit_cb,
				      XmVaRADIOBUTTON, b1, 0, NULL, NULL,
				      XmVaRADIOBUTTON, b2, 0, NULL, NULL,
				      XmVaRADIOBUTTON, b3, 0, NULL, NULL,
					  XmNhighlightThickness, 0,
					  NULL);

    XmStringFree(b1);
    XmStringFree(b2);
    XmStringFree(b3);
    nunits = unit;
    XtManageChild(toggle_box);

    b1 = XmStringCreateSimple(" Be nice to color map");
    tb = XtVaCreateManagedWidget("bn", xmToggleButtonWidgetClass, bul1,
				 XmNlabelString, b1,
				 XmNset, options.nicecol,
				 XmNselectColor, get_pixel(bul1, "red"),
				 XmNx, 16,
				 XmNy, 134,
				 NULL);

    XmStringFree(b1);
    XtAddCallback(tb, XmNvalueChangedCallback, (XtCallbackProc) nncol_cb, NULL);
    nncol = options.nicecol;
}

void do_select_dialog(Widget w, XtPointer client_data, XtPointer call_data)
{
    Pixel foreground;

    if (!selectdialog) {

	foreground = get_pixel(w, "Black");
	XtVaGetValues(w, XmNbackground, &grey, NULL);
    if (options.nicecol)
	background = get_pixel(draw_w, "white");
    else
	background = get_pixel(draw_w, "Snow");

	selectdialog = XtVaCreatePopupShell("selectdialog",
				xmDialogShellWidgetClass, GetTopShell(w),
					    XmNtitle, "Preferences",
					    XmNdeleteResponse, XmUNMAP,
					  XmNresizePolicy, XmRESIZE_NONE,
					    XmNnoResize, True,
					    XmNwidth, 340,
					    XmNminWidth, 340,
					    XmNmaxWidth, 340,
					    XmNheight, 270,
					    XmNminHeight, 270,
					    XmNmaxHeight, 270,
					    XmNmarginWidth, 2,
					    XmNmarginHeight, 2,
					    NULL);

	dialog = XtVaCreateWidget("dialog",
				xmBulletinBoardWidgetClass, selectdialog,
				  XmNforeground, foreground,
				  XmNautoUnmanage, False,
				  XmNdialogStyle, XmDIALOG_WORK_AREA,
				  NULL);
	XtAddCallback(dialog, XmNunmapCallback, (void *) cancel_cb, NULL);

	tab = XtVaCreateWidget("Tab", xmTabWidgetClass, dialog,
			       XmNx, 0,
			       XmNy, 0,
			       NULL);

	bul1 = XtVaCreateWidget("Display",
#ifdef GOOD_GOMETRY_MANAGER
				xmBulletinBoardWidgetClass, tab,
#else
				xmDrawingAreaWidgetClass, tab,
#endif
				XmNforeground, foreground,
				XmNautoUnmanage, False,
				NULL);
	XtManageChild(bul1);
	do_display(bul1);

	bul2 = XtVaCreateWidget("Printing",
				xmBulletinBoardWidgetClass, tab,
				XmNforeground, foreground,
				XmNautoUnmanage, False,
				NULL);
	do_printing(bul2);

	bul3 = XtVaCreateWidget("GPS",
				xmBulletinBoardWidgetClass, tab,
				XmNforeground, foreground,
				XmNautoUnmanage, False,
				NULL);
	do_gps(bul3);

	bul4 = XtVaCreateWidget("Map Settings",
				xmBulletinBoardWidgetClass, tab,
				XmNforeground, foreground,
				XmNautoUnmanage, False,
				NULL);
	do_datum(bul4);

	bul5 = XtVaCreateWidget("Routes",
				xmBulletinBoardWidgetClass, tab,
				XmNforeground, foreground,
				XmNautoUnmanage, False,
				NULL);
	do_routes(bul5);

	cancel = XtVaCreateManagedWidget("Cancel",
					 xmPushButtonWidgetClass, dialog,
					 XmNforeground, foreground,
					 XmNhighlightThickness, 1,
					 XmNhighlightColor, foreground,
					 XmNwidth, 60,
					 XmNx, 140,
					 XmNy, OCHX,
					 NULL);
	XtAddCallback(cancel, XmNactivateCallback, (XtCallbackProc) cancel_cb, NULL);

	start = XtVaCreateManagedWidget("OK",
					xmPushButtonWidgetClass, dialog,
					XmNforeground, foreground,
					XmNhighlightThickness, 1,
					XmNhighlightColor, foreground,
					XmNwidth, 60,
					XmNx, 40,
					XmNy, OCHX,
					NULL);
	XtAddCallback(start, XmNactivateCallback, (XtCallbackProc) ok_cb, NULL);

	help = XtVaCreateManagedWidget("Help",
				       xmPushButtonWidgetClass, dialog,
				       XmNforeground, foreground,
				       XmNhighlightThickness, 1,
				       XmNhighlightColor, foreground,
				       XmNwidth, 60,
				       XmNx, 240,
				       XmNy, OCHX,
				       NULL);
	XtAddCallback(help, XmNactivateCallback, (XtCallbackProc) show_help_cb, "preferences.html");

	XtManageChild(tab);
    }
    XtManageChild(dialog);
    XmTabSetTabWidget(tab, bul1, True);
    XtManageChild(selectdialog);
    XtPopup(selectdialog, XtGrabNone);
}

void popup_printer_tab()
{
    do_select_dialog(toplevel, NULL, NULL);
    XmTabSetTabWidget(tab, bul2, False);
}

